import { __decorate, __metadata } from 'tslib';
import { Injectable, Component, NgModule, ErrorHandler } from '@angular/core';
import { MessageService } from 'primeng/components/common/messageservice';
import { RouterModule } from '@angular/router';
import { hasIn, isFunction, isEmpty, has, isArray } from 'lodash';
import { HttpErrorResponse } from '@angular/common/http';

let ErrorHandlerService = class ErrorHandlerService {
    constructor(messageService) {
        this.messageService = messageService;
        this._console = console;
    }
    handleError(error) {
        const errorLogger = this.getErrorLogger(error);
        const original = this.getOriginalError(error);
        const context = this.getDebugContext(error);
        const message = (original || error).message || `${error}`;
        errorLogger(this._console, 'ERROR', message);
        if (original) {
            errorLogger(this._console, 'ORIGINAL ERROR', original);
        }
        if (context) {
            errorLogger(this._console, 'CONTEXT', context);
        }
        this.messageService.add({
            severity: 'error',
            summary: (original || error).constructor.name,
            detail: message,
            life: 15,
            closable: true,
            data: error,
        });
    }
    getDebugContext(error) {
        return error
            ? error.ngDebugContext ||
                this.getDebugContext(this.getOriginalError(error))
            : null;
    }
    getOriginalError(error) {
        let original = error;
        while (original) {
            original = original.ngOriginalError;
        }
        return original;
    }
    getErrorLogger(error) {
        return error.ngErrorLogger || this.defaultErrorLogger;
    }
    defaultErrorLogger(console, ...values) {
        console.error(...values);
    }
};
ErrorHandlerService.ctorParameters = () => [
    { type: MessageService }
];
ErrorHandlerService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [MessageService])
], ErrorHandlerService);

let GenericErrorComponent = class GenericErrorComponent {
};
GenericErrorComponent = __decorate([
    Component({
        selector: 'lib-generic-error',
        template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Erreur !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">Une erreur s'est produite !</p>\n</div>\n"
    })
], GenericErrorComponent);

let NotFoundErrorComponent = class NotFoundErrorComponent {
};
NotFoundErrorComponent = __decorate([
    Component({
        selector: 'lib-not-found-error',
        template: "<div class=\"card ui-message ui-messages-error\">\n  <h1>\n    <i class=\"ui-messages-icon fa fa-exclamation-triangle\"></i>\n    <span i18n class=\"ui-messages-summary\">Page non trouv\u00E9e !</span>\n  </h1>\n  <p i18n class=\"ui-messages-detail\">La ressource n'a pas \u00E9t\u00E9 trouv\u00E9e.</p>\n</div>\n"
    })
], NotFoundErrorComponent);

const routes = [
    {
        path: 'error/404',
        pathMatch: 'full',
        component: NotFoundErrorComponent,
    },
    { path: 'error/**', component: GenericErrorComponent },
];
let ErrorModule = class ErrorModule {
};
ErrorModule = __decorate([
    NgModule({
        imports: [RouterModule.forChild(routes)],
        exports: [RouterModule],
        providers: [{ provide: ErrorHandler, useClass: ErrorHandlerService }],
        declarations: [GenericErrorComponent, NotFoundErrorComponent],
    })
], ErrorModule);

function isErrorMessageHandler(that) {
    return hasIn(that, 'setError') && isFunction(that.setError);
}
function isConstraintViolationHandler(that) {
    return (hasIn(that, 'setConstraintViolations') &&
        isFunction(that.setConstraintViolations));
}
function isValidationErrorHandler(that) {
    return hasIn(that, 'setErrors') && isFunction(that.setErrors);
}
function isErrorLogger(that) {
    return hasIn(that, 'logError') && isFunction(that.logError);
}
function isErrorHandler(that) {
    return (isErrorMessageHandler(that) ||
        isConstraintViolationHandler(that) ||
        isValidationErrorHandler(that) ||
        isErrorLogger(that));
}
function isErrorClearer(that) {
    return hasIn(that, 'clearErrors') && isFunction(that.clearErrors);
}
function isDispatchableError(that) {
    return hasIn(that, 'dispatch') && isFunction(that.dispatch);
}

/**
 * Erreur "enveloppant" une autre erreur.
 */
class AppError {
    constructor(original, message, stack, transient = false) {
        this.original = original;
        this.message = message;
        this.stack = stack;
        this.transient = transient;
        this.isNullError = false;
    }
    get name() {
        return this.getName();
    }
    dispatch(handler) {
        return ((isErrorLogger(handler) &&
            handler.logError(this.message, this.name, this.stack)) ||
            (isErrorMessageHandler(handler) && handler.setError(this.message)) ||
            (isValidationErrorHandler(handler) &&
                handler.setErrors({ message: this.message })));
    }
    getName() {
        return ((this.original instanceof Error && this.original.name) ||
            this.constructor.name);
    }
}

/**
 * Supprime les erreurs d'un handler.
 */
function clearErrors(handler) {
    if (isErrorClearer(handler) && handler.clearErrors()) {
        return true;
    }
    let handled = false;
    if (isErrorMessageHandler(handler) && handler.setError(null)) {
        handled = true;
    }
    if (isValidationErrorHandler(handler) && handler.setErrors(null)) {
        handled = true;
    }
    if (isConstraintViolationHandler(handler) &&
        handler.setConstraintViolations(null)) {
        handled = true;
    }
    return handled;
}

/**
 * Erreur basée sur une réponse HTTP d'erreur.
 */
class HTTPError extends AppError {
    constructor(original, message) {
        super(original, message || original.message || original.statusText, undefined, original.status >= 500);
    }
    get status() {
        return this.original.status;
    }
    get statusText() {
        return this.original.statusText;
    }
    get headers() {
        return this.original.headers;
    }
    get url() {
        return this.original.url;
    }
    get error() {
        return this.original.error;
    }
}
/**
 * Erreur 403: Forbidden
 */
class AccessDeniedError extends HTTPError {
}
/**
 * Erreur 401: Authentication Required
 */
class AuthenticationError extends HTTPError {
}
/**
 * Erreur 400: Bad Request
 */
class BadRequestError extends HTTPError {
}

/**
 * Erreur de validation retournée par api-platform.
 */
class ConstraintViolations extends BadRequestError {
    constructor(original, message, violations) {
        super(original, message);
        this.violations = violations;
    }
    dispatch(handler) {
        return ((isConstraintViolationHandler(handler) &&
            handler.setConstraintViolations(this.violations)) ||
            super.dispatch(handler));
    }
    static create(violations) {
        return new ConstraintViolations(new HttpErrorResponse({
            status: 400,
            statusText: 'Constraint violations',
        }), 'Constraint violations', violations);
    }
}

const ɵ0 = (handler) => {
    return clearErrors(handler);
};
/**
 * Singleton d'erreur null.
 */
const NullError = {
    isNullError: true,
    name: 'NullError',
    message: 'No error',
    dispatch: ɵ0,
};

/**
 * Transforme une valeur en erreur.
 *
 * Les informations intéressantes d'HttpErrorResponse et Error sont conservés.
 */
function normalizeError(err) {
    if (err instanceof AppError || err === NullError) {
        return err;
    }
    if (err instanceof HttpErrorResponse) {
        return normalizeHttpErrorResponse(err);
    }
    if (err instanceof Error) {
        return new AppError(err, err.message, err.stack, false);
    }
    if (isEmpty(err)) {
        return NullError;
    }
    if (typeof err === 'object') {
        return new AppError(err, extractMessage(err), err.stack, err.transient || false);
    }
    return new AppError(err, extractMessage(err));
}
function normalizeHttpErrorResponse(err) {
    const body = 'error' in err ? err.error : {};
    const message = extractMessage(body) || extractMessage(err);
    switch (err.status) {
        case 400:
            if (has(body, 'violations') && isArray(body.violations)) {
                return new ConstraintViolations(err, message, body.violations);
            }
            return new BadRequestError(err, message);
        case 401:
            return new AuthenticationError(err, message);
        case 403:
            return new AccessDeniedError(err, message);
        default:
            return new HTTPError(err, message);
    }
}
function extractMessage(err) {
    if (!err) {
        return undefined;
    }
    switch (typeof err) {
        case 'object':
            if ('message' in err) {
                return err.message;
            }
            if ('hydra:description' in err) {
                return err['hydra:description'];
            }
            if ('hydra:title' in err) {
                return err['hydra:title'];
            }
            if ('error' in err) {
                return extractMessage(err.error);
            }
            break;
        case 'string':
        case 'number':
            return `${err}`;
    }
    return undefined;
}

/**
 * Erreur émise lorsqu'une exception ne peut pas être dispatchée.
 */
class UnhandledError extends AppError {
    constructor(original) {
        super(original, `Unhandled error: ${original.message}`, original.stack, false);
    }
}

class ValidationError extends AppError {
    constructor(errors) {
        super(errors, errors.message || 'Validation error');
        this.errors = errors;
    }
    dispatch(handler) {
        return ((isValidationErrorHandler(handler) && handler.setErrors(this.errors)) ||
            super.dispatch(handler));
    }
}

/*
 * Public API Surface of error-types
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AccessDeniedError, AppError, AuthenticationError, BadRequestError, ConstraintViolations, ErrorHandlerService, ErrorModule, GenericErrorComponent, HTTPError, NotFoundErrorComponent, NullError, UnhandledError, ValidationError, clearErrors, isConstraintViolationHandler, isDispatchableError, isErrorClearer, isErrorHandler, isErrorLogger, isErrorMessageHandler, isValidationErrorHandler, normalizeError, ɵ0 };
//# sourceMappingURL=devatscience-ngx-errors.js.map
